Attribute VB_Name = "MiscFunc"
' The Miscellaneous Functions Latch.
' Read the code or the FS service manual to know what the
' miscellaneous funky latch actually is there for... It's
' an amusing cross between nifty economisation and spooky
' cruftiness. :-)
'
' Project: FileStore [E01S emulator]
' Module : MiscFunc
'
' Version: 0.00
' Date   : 19th April 2009
' Started: 19th April 2009
'
' Author : Rick Murray, heyrick1973 -at- yahoo -dot- co -dot- uk
'          http://www.heyrick.co.uk/econet/
'
'
' Licence:
'
'   Copyright 2010 Rick Murray
'
'   Licensed under the EUPL, Version 1.1 ONLY (the "Licence");
'   You may not use this work except in compliance with the Licence.
'
'   You may obtain a copy of the Licence at:
'
'     http://ec.europa.eu/idabc/eupl5
'
'   Unless required by applicable law or agreed to in writing, software
'   distributed under the Licence is distributed on an "AS IS" basis,
'   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
'   implied. See the Licence for the specific language governing
'   permissions and limitations under the Licence.
'
'
'   Rick's note: The EUPL is "compatible" with GPL v2, but *NOT* the more
'                restrictive GPL v3. If you will be involving GPL code
'                in this project, or using some of this code in a GPL
'                project, you should specify the use of ONLY the GPL v2
'                (and not the boilerplate "GPL 2 or any later...").
'

Option Explicit

Public Type LatchDef
  Floppy1 As Boolean              ' Accessing Floppy1?
  Floppy2 As Boolean              ' Accessing Floppy2?
  FloppySide As Boolean           ' Which 'side' of the floppy?
  RTCOkay As Boolean              ' Accessing the NVRAM chip?
  FDCDiscDensity As Boolean       ' Sets the FDC "DDEN" pin.
  FDCMasteReset As Boolean        ' Sets the FDC "MRST" pin.
  FDCTest As Boolean              ' Sets the FDC "TEST" pin.
  RedLED As Boolean               ' Turns the "Mode" LED on/off.
End Type

' IMPORTANT: The three FDCxxx signals are ACTIVE LOW, so we
'            invert the signal in code here; so if FDCTest is
'            True, that means we want test mode.
'            [where in reality '0' is Test, '1' is normal]

Public Latch As LatchDef
Public LatchByte As Byte

Private PrevLED As Boolean



Public Sub Latch_Write(ByVal Value As Byte)
  ' Decode the input
  Latch.Floppy1 = ((Value And 1) = 1)
  Latch.Floppy2 = ((Value And 2) = 2)
  Latch.FloppySide = ((Value And 4) = 4)
  Latch.RTCOkay = ((Value And 8) = 8)              ' \/\/\/\/
  Latch.FDCDiscDensity = Not ((Value And 16) = 16) ' INVERTED
  Latch.FDCMasteReset = Not ((Value And 32) = 32)  ' INVERTED
  Latch.FDCTest = Not ((Value And 64) = 64)        ' INVERTED
  Latch.RedLED = ((Value And 128) = 128)           ' /\/\/\/\

  ' If something FDC-related changes, notify the FDC hardware
  ' The 119 masks the FDC bits: %01110111
  ' We notify FDC for drive/side which isn't exactly required by
  ' the 2793, but will be necessary as the same code will be
  ' handling the floppy disc image reading.
  ' (I'm totally surprised that the FDC itself isn't telling us
  '  which drive/side to be reading; I bet the FileStore's disc
  '  access code was an absolute joy to debug! :-) )
  If ((Value And 119) <> (LatchByte And 119)) Then
    FDC_ChangeState
  End If
  
  ' We don't bother doing anything with the RTC bit - this will
  ' be checked upon any attempts to read/write the NVRAM.
  
  ' We can do the red LED directly from here
  If (Latch.RedLED = True) Then
    If (PrevLED <> True) Then HWLog "Mode LED on"
    frmMain.lblMode.Visible = True
    frmMain.lblCMode.Visible = True
  Else
    If (PrevLED <> False) Then HWLog "Mode LED off"
    frmMain.lblMode.Visible = False
    frmMain.lblCMode.Visible = False
  End If
  PrevLED = Latch.RedLED
  
  ' remember this for the next time (FDC change logging)
  LatchByte = Value
End Sub
