Attribute VB_Name = "Setup"
' Setup/config code.
' Handles emulator setup and configuration.
'
' Project: FileStore [E01S emulator]
' Module : Setup
'
' Version: 0.00
' Date   : 30th July 2009
' Started: 11th April 2009
'
' Author : Rick Murray, heyrick1973 -at- yahoo -dot- co -dot- uk
'          http://www.heyrick.co.uk/econet/
'
'
' Licence:
'
'   Copyright 2010 Rick Murray
'
'   Licensed under the EUPL, Version 1.1 ONLY (the "Licence");
'   You may not use this work except in compliance with the Licence.
'
'   You may obtain a copy of the Licence at:
'
'     http://ec.europa.eu/idabc/eupl5
'
'   Unless required by applicable law or agreed to in writing, software
'   distributed under the Licence is distributed on an "AS IS" basis,
'   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
'   implied. See the Licence for the specific language governing
'   permissions and limitations under the Licence.
'
'
'   Rick's note: The EUPL is "compatible" with GPL v2, but *NOT* the more
'                restrictive GPL v3. If you will be involving GPL code
'                in this project, or using some of this code in a GPL
'                project, you should specify the use of ONLY the GPL v2
'                (and not the boilerplate "GPL 2 or any later...").
'

Option Explicit

Public Type SetupDef
  IDword As String * 4
  Version As Integer
  DriveImage(0 To 5) As String   ' for :0, :4, :5
  DriveLocked(4 To 5) As Boolean ' write protected?
  ROMImage As String             ' Path to ROM image file
  
  StartFlapOpen As Boolean
  StartNVRAMClear As Boolean
  StartHalted As Boolean
  
  ' some predefs for the future
  ReservedFlag3 As Boolean
  ReservedFlag4 As Boolean
  ReservedFlag5 As Boolean
  ReservedFlag6 As Boolean
  ReservedFlag7 As Boolean
  ReservedInt0 As Integer
  ReservedInt1 As Integer
  ReservedInt2 As Integer
  ReservedInt3 As Integer
End Type
Public SetupInfo As SetupDef

Public FlapOpen As Boolean



Public Sub Setup_LoadOpts()
  Dim FP As Integer, IDword As String, IDvers As Integer
  
  If (Len(Dir(App.Path & "\settings.dat")) = 0) Then
LoadJump:
    ' set up a default configuration
    SetupInfo.DriveImage(0) = ""
    If (Len(Dir(App.Path & "\bootdisc.img")) = 0) Then
      SetupInfo.DriveImage(4) = "<AppPath>\fsdisc.img"
    Else
      SetupInfo.DriveImage(4) = ""
    End If
    SetupInfo.DriveImage(5) = ""
    SetupInfo.DriveLocked(4) = False
    SetupInfo.DriveLocked(5) = False
    
    SetupInfo.ROMImage = "<AppPath>\e01_v131.dat"
    
    SetupInfo.StartFlapOpen = False
    SetupInfo.StartNVRAMClear = False
    SetupInfo.StartHalted = False
    
    Exit Sub
  End If
  
  FP = FreeFile
  Open App.Path & "\settings.dat" For Binary Access Read As FP
  
  IDword = "****"
  Get #FP, , IDword
  If (IDword <> "FSCF") Then
    MsgBox "Configuration file ID word is not valid.", _
           vbExclamation + vbOKOnly
    Close #FP
    GoTo LoadJump
  End If
  
  Get #FP, , IDvers
  
  If (IDvers = 2) Then
    ' Normally we would bend over backwards to support previous
    ' versions of configuration file, but as this software is
    ' not fully operational, we won't bother...
    ' Version #2 defined:
    '   SetupInfo.ROMImage(1 To 2) As String
    ' Version #3 defined:
    '   SetupInfo.StartHalted
    '   and the ReservedXXX pre-defs
    Close #FP
    MsgBox "Your configuration file cannot be loaded as the " & _
           "format has changed." & vbCrLf & _
           "Please reconfigure as appropriate.", _
           vbExclamation + vbOKOnly
    GoTo LoadJump
  
  ElseIf (IDvers <> 3) Then
    Close #FP
    MsgBox "Configuration file version is not valid." & vbCrLf & _
           "Your settings have been restored to defaults. " & _
           "Please reconfigure as appropriate.", _
           vbExclamation + vbOKOnly
    GoTo LoadJump
  End If
  
  Get #FP, 1, SetupInfo
  Close #FP
  
  ' ##TODO##
  ' here we force the drives to be write protected
  SetupInfo.DriveLocked(4) = True
  SetupInfo.DriveLocked(5) = True
End Sub


Private Sub Setup_SaveOpts()
  Dim FP As Integer, MyFile As String
  
  MyFile = App.Path & "\settings.dat"
  
  If (Len(Dir(App.Path & "\settings.dat")) > 0) Then
    Kill MyFile
  End If
  
  SetupInfo.IDword = "FSCF" ' FileStore Configuration File
  SetupInfo.Version = 3     ' version 3 now
  FP = FreeFile
  Open MyFile For Binary Access Write As FP
  Put #FP, , SetupInfo
  Close #FP
End Sub


Public Sub Setup_PullOpts()
  ' Read configuration from frmSetup.
  
  ' Disc image files
  SetupInfo.DriveImage(0) = _
    Setup_ParseFile(frmSetup.txtImgFile(2).Text)
  SetupInfo.DriveImage(4) = _
    Setup_ParseFile(frmSetup.txtImgFile(0).Text)
  SetupInfo.DriveImage(5) = _
    Setup_ParseFile(frmSetup.txtImgFile(1).Text)

  ' Disc write protected? (:4 and :5 only)
  SetupInfo.DriveLocked(4) = _
    IIf((frmSetup.optImgWriteProtected(0).Value = 1), True, False)
  SetupInfo.DriveLocked(5) = _
    IIf((frmSetup.optImgWriteProtected(1).Value = 1), True, False)
  
  ' EPROM images
  SetupInfo.ROMImage = _
    Setup_ParseFile(frmSetup.txtImgFile(3).Text)

  ' Startup options
  SetupInfo.StartFlapOpen = _
    IIf((frmSetup.optStartOpt(0).Value = 1), True, False)
  SetupInfo.StartNVRAMClear = _
    IIf((frmSetup.optStartOpt(1).Value = 1), True, False)
  SetupInfo.StartHalted = _
    IIf((frmSetup.optStartOpt(2).Value = 1), True, False)
    
  ' Save it
  Setup_SaveOpts
End Sub


Public Sub Setup_PushOpts()
  ' Write current configuration to frmSetup.
  
  ' Disc image files
  frmSetup.txtImgFile(2).Text = SetupInfo.DriveImage(0)
  frmSetup.txtImgFile(0).Text = SetupInfo.DriveImage(4)
  frmSetup.txtImgFile(1).Text = SetupInfo.DriveImage(5)
  
  ' Disc write protected? (:4 and :5 only)
  frmSetup.optImgWriteProtected(0).Value = _
    IIf((SetupInfo.DriveLocked(4) = True), 1, 0)
  frmSetup.optImgWriteProtected(1).Value = _
    IIf((SetupInfo.DriveLocked(5) = True), 1, 0)
      
  ' EPROM images
  frmSetup.txtImgFile(3).Text = SetupInfo.ROMImage
  
  ' Startup options
  frmSetup.optStartOpt(0).Value = _
    IIf((SetupInfo.StartFlapOpen = True), 1, 0)
  frmSetup.optStartOpt(1).Value = _
    IIf((SetupInfo.StartNVRAMClear = True), 1, 0)
  frmSetup.optStartOpt(2).Value = _
    IIf((SetupInfo.StartHalted = True), 1, 0)
End Sub


Public Function Setup_ParseFile(ByVal FName As String) As String
  ' We try to match the application path, and if it is
  ' present, we replace it with "<AppPath>". This makes the
  ' eventual display tidier:
  ' Compare:
  '   <AppPath>\bootdisc.img
  ' with:
  '   C:\Program Files\HeyRick\FileStore\bootdisc.img
  ' or worse:
  '   C:\Program Files\DevStudio\My Applications\FileStore\boo...
  
  Dim OurPath As String, OurLen As Integer, NewPath As String
  
  OurPath = App.Path
  OurLen = Len(OurPath)
  
  If (Left$(FName, OurLen) = OurPath) Then
    NewPath = "<AppPath>" & Mid$(FName, (OurLen + 1))
  Else
    NewPath = FName
  End If
  
  Setup_ParseFile = NewPath
End Function


Public Function Setup_RebuildFile(FName As String) As String
  ' If the leftmost part of the input is "<AppPath>", replace
  ' it with our own application folder...
    
  If (LCase$(Left$(FName, 9)) = "<apppath>") Then
    Setup_RebuildFile = App.Path & Mid$(FName, 10)
  Else
    Setup_RebuildFile = FName
  End If
End Function

