Attribute VB_Name = "VIAmod"
' 6522 VIA
'
' Project: FileStore [E01S emulator]
' Module : VIAmod
'
' Version: 0.00
' Date   : 20th April 2009
' Started: 20th April 2009
'
' Author : Rick Murray, heyrick1973 -at- yahoo -dot- co -dot- uk
'          http://www.heyrick.co.uk/econet/
'
'
' Licence:
'
'   Copyright 2010 Rick Murray
'
'   Licensed under the EUPL, Version 1.1 ONLY (the "Licence");
'   You may not use this work except in compliance with the Licence.
'
'   You may obtain a copy of the Licence at:
'
'     http://ec.europa.eu/idabc/eupl5
'
'   Unless required by applicable law or agreed to in writing, software
'   distributed under the Licence is distributed on an "AS IS" basis,
'   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
'   implied. See the Licence for the specific language governing
'   permissions and limitations under the Licence.
'
'
'   Rick's note: The EUPL is "compatible" with GPL v2, but *NOT* the more
'                restrictive GPL v3. If you will be involving GPL code
'                in this project, or using some of this code in a GPL
'                project, you should specify the use of ONLY the GPL v2
'                (and not the boilerplate "GPL 2 or any later...").
'

Option Explicit


Private Const ORB  As Integer = 0
Private Const IRB  As Integer = 0
Private Const ORAH As Integer = 1 ' handshaking version, or else ORA 15
Private Const DDRB As Integer = 2
Private Const DDRA As Integer = 3
Private Const T1CL As Integer = 4
Private Const T1CH As Integer = 5
Private Const T1LL As Integer = 6
Private Const T1LH As Integer = 7
Private Const T2CL As Integer = 8
Private Const T2CH As Integer = 9
Private Const SR   As Integer = 10
Private Const ACR  As Integer = 11
Private Const PCR  As Integer = 12
Private Const IFR  As Integer = 13
Private Const IER  As Integer = 14
Private Const ORA  As Integer = 15
Private Const IRA As Integer = 15


Private Type Portdef
  ' definition for port data
  Data As Integer   ' 8 bits of data
  LDat As Integer   ' Latched data (port B)
  DDir As Integer   ' Data direction flags
  Ctrl As Integer   ' CA1/CA2 or CB1/CB2
End Type

Private Type PCRdef
  ' definition for Peripheral Control Register
  CA1Ctrl As Boolean  ' 1 bit
  CA2Ctrl As Integer  ' 3 bits
  CB1Ctrl As Boolean  ' 1 bit
  CB2Ctrl As Integer  ' 3 bits
End Type

Private Type ACRdef
  ' definition for the Auxiliary Control Register
  PALatch As Boolean
  PBLatch As Boolean
  SRCtrl  As Integer  ' 3 bits, but Shift Register not implemented
  T2Ctrl  As Boolean
  T1Ctrl  As Integer  ' 2 bits
End Type

Private Type IFRdef
  ' definition for the Interrupt Flag Register
  CA2Int  As Boolean
  CA1Int  As Boolean
  SRInt   As Boolean
  CB2Int  As Boolean
  CB1Int  As Boolean
  Ti2Int  As Boolean
  Ti1Int  As Boolean
  IRQ     As Boolean
End Type

Private Type IERdef
  ' definition for the Interrupt Enable Register
  CA2     As Boolean
  CA1     As Boolean
  SR      As Boolean
  CB2     As Boolean
  CB1     As Boolean
  Ti2     As Boolean
  Ti1     As Boolean
  Set     As Boolean
End Type

Private Type VIAdef
  ' definition for the VIA
  PortA As Portdef
  PortB As Portdef
  
  Timer1 As Long
  Timer1Latch As Long
  
  ' Timer2 not currently supported
  
  PCR As PCRdef
  ACR As ACRdef
  IFR As IFRdef
  IER As IERdef
End Type

Private VIA As VIAdef


Public Sub VIA_Initialise()
  ' does nothing yet
  
End Sub


Public Sub VIA_Write(ByVal Value As Byte, ByVal Reg As Integer)
  ' stats
  
  HWLog "VIA write: " & CStr(Value) & " -> " & CStr(Reg) & _
        " [" & VIA_LookupReg(Reg) & "  %" & VIA_Binary(Value) & "]"
End Sub


Public Function VIA_Read(ByVal Reg As Integer)
  ' stats
  
  HWLog "VIA read : <- " & CStr(Reg) & " [" & _
        VIA_LookupReg(Reg) & "]"
End Function


Public Sub VIA_Poll()
  ' does nothing
  
End Sub


Private Function VIA_LookupReg(ByVal Reg As Integer) As String
  Select Case Reg
    Case 0: VIA_LookupReg = "IORB"
    Case 1: VIA_LookupReg = "IORAH"
    Case 2: VIA_LookupReg = "DDRB"
    Case 3: VIA_LookupReg = "DDRA"
    Case 4: VIA_LookupReg = "T1CL"
    Case 5: VIA_LookupReg = "T1CH"
    Case 6: VIA_LookupReg = "T1LL"
    Case 7: VIA_LookupReg = "T1LH"
    Case 8: VIA_LookupReg = "T2CL"
    Case 9: VIA_LookupReg = "T2CH"
    Case 10: VIA_LookupReg = "SR"
    Case 11: VIA_LookupReg = "ACR"
    Case 12: VIA_LookupReg = "PCR"
    Case 13: VIA_LookupReg = "IFR"
    Case 14: VIA_LookupReg = "IER"
    Case 15: VIA_LookupReg = "IORA-"
  End Select
End Function


Public Function VIA_Binary(ByVal Value As Integer) As String
  ' Crappy hacky code to convert a number to binary so I can
  ' check what's going on against my VIA datasheet...

  Dim Recurse As Integer, WorkVal As String
  
  ' Set up initial values
  WorkVal = String(8, "-")

  For Recurse = 7 To 0 Step -1
    If ((Value And (2 ^ Recurse)) <> 0) Then
      Mid(WorkVal, (8 - Recurse), 1) = "1"
    Else
      Mid(WorkVal, (8 - Recurse), 1) = "0"
    End If
  Next

  VIA_Binary = WorkVal
End Function
