Attribute VB_Name = "Wrapper"
' Wrapper. Sub Main, misc functions, and the emulation loop...
'
' Project: FileStore [E01S emulator]
' Module : Wrapper
'
' Version: 0.00
' Date   : 8th August 2010
' Started: 11th April 2009
'
' Author : Rick Murray, heyrick1973 -at- yahoo -dot- co -dot- uk
'          http://www.heyrick.co.uk/econet/
'
'
' Licence:
'
'   Copyright 2010 Rick Murray
'
'   Licensed under the EUPL, Version 1.1 ONLY (the "Licence");
'   You may not use this work except in compliance with the Licence.
'
'   You may obtain a copy of the Licence at:
'
'     http://ec.europa.eu/idabc/eupl5
'
'   Unless required by applicable law or agreed to in writing, software
'   distributed under the Licence is distributed on an "AS IS" basis,
'   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
'   implied. See the Licence for the specific language governing
'   permissions and limitations under the Licence.
'
'
'   Rick's note: The EUPL is "compatible" with GPL v2, but *NOT* the more
'                restrictive GPL v3. If you will be involving GPL code
'                in this project, or using some of this code in a GPL
'                project, you should specify the use of ONLY the GPL v2
'                (and not the boilerplate "GPL 2 or any later...").
'

Option Explicit

Private Declare Sub Sleep Lib "Kernel32" _
                (ByVal dwMilliseconds As Long)

Private Declare Function ShellExecute Lib "Shell32" _
                Alias "ShellExecuteA" _
               (ByVal hwnd As Long, _
                ByVal lpOperation As String, _
                ByVal lpFile As String, _
                ByVal lpParameters As String, _
                ByVal lpDirectory As String, _
                ByVal nShowCmd As Long) As Long

Private Declare Function GetComputerNameA Lib "Kernel32" _
               (ByVal lpBuffer As String, _
                ByVal lpnSize As String) As Boolean


Private Const SW_NORMAL As Integer = 1

Public AppVers As String
Public AppDate As String

Private ShutdownInterlock As Boolean


' logging hardware access - for build 2009/04/20++ to see what
' hardware is accessed, and when
Public HWFP As Integer


Private Sub Main()
  ' All we do here is load the setup form and await
  ' the user clicking on "Start the server"...
  
  AppVers = CStr(App.Major) & "." & _
            Right$("00" & CStr(App.Minor), 2)
  AppDate = "2010/08/08"
  
  Setup_LoadOpts
  
  If ((InStr(1, Command$, "-help") > 0) Or (InStr(1, Command$, "-?") > 0)) Then
    MsgBox "FileStore emulator, command line options:" & vbCrLf & vbCrLf & _
           vbTab & "-start" & vbTab & "Auto-start the emulator (no setup)" & _
           vbCrLf & vbCrLf & _
           vbTab & "-halt" & vbTab & "Begin with execution halted" & vbCrLf & _
           vbTab & "-run" & vbTab & "Begin normal execution" & vbCrLf & vbCrLf & _
           vbTab & "-open" & vbTab & "Begin with front flap open" & vbCrLf & _
           vbTab & "-closed" & vbTab & "Begin wth front flap closed" & _
           vbCrLf & vbCrLf & "Copyright  2009-2010 Rick Murray" & vbCrLf & _
           vbCrLf & "The source code to this software is available." & _
           vbCrLf & "For more information, updates, source, etc, please visit:" & _
           vbCrLf & vbTab & "http://www.heyrick.co.uk/econet/fs/emulator.html", _
           vbInformation, "FileStore E01S emulator " & AppVers & " (" & _
           AppDate & ")"
    End
  End If
  
  If (InStr(1, Command$, "-run", vbTextCompare) > 0) Then
    SetupInfo.StartHalted = False
    ExecHalt = False
  End If
  
  If (InStr(1, Command$, "-halt", vbTextCompare) > 0) Then
    SetupInfo.StartHalted = True
    ExecHalt = True
  End If
  
  If (InStr(1, Command$, "-open", vbTextCompare) > 0) Then
    SetupInfo.StartFlapOpen = True
    FlapOpen = True
  End If
  
  If (InStr(1, Command$, "-closed", vbTextCompare) > 0) Then
    SetupInfo.StartFlapOpen = False
    FlapOpen = False
  End If
  
  If (InStr(1, Command$, "-start", vbTextCompare) > 0) Then
    StartServer
  Else
    Load frmSetup
    frmSetup.Show
  End If
End Sub


Public Sub StartServer()
  ' When the setup has been done and "Start the server" clicked,
  ' we come to here. This gets it all going and runs the outer
  ' loop of the emulation.
  
  Dim Recurse As Integer, STick As Integer, PollDelay As Long
  
  ' Display the splash
  Load frmInit
  frmInit.lblVersion = "Version " & AppVers & " (" & _
    AppDate & " UNFINISHED) by Rick Murray"
  frmInit.lblLicence.Caption = "Licenced under the " & _
    "EUPL v1.1 (open source). Refer to licence for Ts&&Cs."
  frmInit.lblLicence.Font.Bold = True
  frmInit.lblLicence.ForeColor = vbRed
  frmInit.lblStatus = "Initialising..."
  frmInit.lblStatus.Refresh
  frmInit.Show
  frmInit.Refresh
  DoEvents
  
  ' Load the forms
  Load frmADLC
  Load frmBreak
  Load frmCPUState
  Load frmDiss
  Load frmMain
  Load frmMem

  ' Start up the hardware log
  HWFP = FreeFile
  If (Len(Dir(App.Path & "\hwlog.txt")) <> 0) Then
    Kill App.Path & "\hwlog.txt"
  End If
  Open App.Path & "\hwlog.txt" For Binary Access Write As HWFP
#If DEMON = 1 Then
  Put #HWFP, , "No logging in speed demon mode..."
#End If
  
  ' Initialise the processor - also initialises the hardware, and
  ' as a consequence (of memory init) loads the EPROM image...
  CPU_Initialise
  
  ' Okay, we're all ready to begin emulation execution
  Unload frmInit
  
  ' Display the FileStore!
  frmMain.Show
  frmMain.Refresh
  DoEvents
  
  ' halted?
  ExecHalt = SetupInfo.StartHalted
  frmMain.SetHaltState

  Do
    If (ExecHalt = False) Then
      For Recurse = 1 To 32
        CPU_Execute True ' 1 to 32 * 128 = 4096 instructions here
      
        ' patch the cycle count to prevent it overflowing
        If (Cycles > &H6FFFFFFF) Then
          Cycles = 0
          PrevCycles = 0
        End If
    
        If (ExecHalt = True) Then Exit For
    
        ' ##TODO## To be honest we MIGHT be polling too often?
        ' Poll the hardware (again, just do it alphabetically...)
        PollDelay = Cycles - PrevCycles ' how long has it been?
        ADLC_Poll
        FDC_Poll PollDelay
        HDC_Poll
        NVRAM_Poll
        VIA_Poll
        PrevCycles = Cycles
      Next ' central execution loop
      
      ' Don't update the caption too often, there's no need.
      frmMain.Caption = "FileStore E01S - " & CPU_State

#If DEMON = 0 Then
      ' Refresh the trace window, but not all the time!
      CPU_RefreshTrace ' (checks it is actually 'visible' too)

      ' Update the memory display, if it is open.
      frmMem.DrawMemPage
#Else
      STick = STick + 1
      If (STick > 7) Then
        ' as before, but less frequently in speed demon mode
        CPU_RefreshTrace
        
        frmMem.DrawMemPage
        
        STick = 0
      End If
#End If

    
    End If ' ExecHalt?
    
    ' Yielding to other processes in what is effectively an
    ' locked loop is a necessity; else you'll see the Windows
    ' implementation of pre-emtive multitasking fall on its ass
    ' as VB does a lot of nothing at the expense of everything
    ' else. This, unfortunately, looks to be an insurmountable
    ' implicit delay. Tant pis, hein?
    Sleep 1 ' shame we can't "Sleep 0"...
    DoEvents
  Loop Until False
End Sub


Public Sub OpenURL(ByVal MyURL As String)
  ' fire off the URL request; DON'T BOTHER FAULTING if an error...

  Dim OurName As String
  
  ' Undocumented, redirect to local server if the name of
  ' the machine is either "AIKO" or "AZUMI".
  ' This is NOT a security risk as it depends on...
  '   1. The local machine having the correct name.
  '   2. It running a webserver as "aiko" or "azumi".
  '   3. That webserver serving the document "/econet/index.html"
  ' Chances are if you are NOT me, you'll flunk at test #1!
  ' :-)
  ' You might think, like, why don't I check to see if I am
  ' online and I can then direct locally or to live server upon
  ' that test? It's simple. Got no landline at home, no ADSL. So
  ' the only way I'll get Aiko/Azumi on-line is via GPRS and have
  ' you, like, SEEN the going price of that over here in France?
  ' It's 15cts/10KiB. Yes, ten sodding Ks will cost you 15 centimes.
  ' Not so bad, until you realise that loading HALF of the home
  ' page of www.bbc.co.uk/news into OperaMini (before I realised
  ' and cut the connection) cost me over THREE euros. So, no, I
  ' don't think *I* will be needing a test like that! Feel free
  ' to add it, or just ^X this chunk of code in your version!
  OurName = String(260, vbNullChar)
  GetComputerNameA OurName, 260
  OurName = Left(OurName, (InStr(1, OurName, vbNullChar) - 1))
  If (OurName = "AIKO") Then _
    MyURL = "http://aiko/econet/index.html"
  If (OurName = "AZUMI") Then _
    MyURL = "http://azumi/econet/index.html"
  ' END of Aiko/Azumi check code. To disable this functionality,
  ' simply comment out the above five lines...


  Screen.MousePointer = vbHourglass
  ShellExecute frmMain.hwnd, "open", MyURL, vbNullString, _
               vbNullString, SW_NORMAL
  Screen.MousePointer = vbDefault
End Sub


Public Sub Shutdown()
  ' Centralised shutdown routinr

  Dim Recurse As Long

  ' Handle interlock case (i.e. we are called from a menu or such
  ' and when unloading the main window it triggers us again...).
  If (ShutdownInterlock = True) Then Exit Sub
  ShutdownInterlock = True

  ' Error? Carry on regardless...
  On Error Resume Next

  ' Save the current contents of the NVRAM for next time
  NVRAM_Save

  ' Unload all of the forms
  For Recurse = Forms.Count - 1 To 0 Step -1
    ' Note that unloading triggers these events:
    '    Form_QueryUnload
    '  & Form_Unload
    Unload Forms(Recurse)
  Next

  ' Close all files that may be open.
  Close

  ' Force an end to this madness...
  End
End Sub


Public Sub HWLog(MyStr As String)
  Dim OutStr As String
  
  ' Format is:
  '   xxxxxxxxxxx &xxxx : xxxxxxxxxxxxxxxxxxxxxxx
  '   {cycle cnt}  {PC}   {blahblah}
  
#If DEMON = 0 Then
  OutStr = Right$(Space(11) & CStr(Cycles), 11) & " &" & _
           Right$("0000" & Hex(CPU_ReadPC), 4) & " : " & _
           MyStr & vbCrLf
  Put #HWFP, , OutStr
#End If
Debug.Print Left$(OutStr, Len(OutStr) - 2)
End Sub


Public Sub ReportFSError()
  ' If an error string has been written to NVRAM, decode and
  ' display it...

  Dim OutStr As String, ErrCode As Integer, ErrAddr As Long
  Dim ErrX As Byte, ErrY As Byte, ErrMsg As String, SINnum As Long
  
  ' +20 = Error code
  ' +21 = X register
  ' +22 = Y register
  ' +23 = Address (MSB)
  ' +24 = Address (LSB)
  ' +25 = 24 bit SIN (not reported at this time ##TODO##)
  
  ErrCode = CInt(NVRAM_ReadByte(20))
  If (ErrCode = 0) Then
    MsgBox "No error has been reported.", vbInformation + _
           vbOKOnly, "FS error report"
    Exit Sub
  End If
  
  ErrX = NVRAM_ReadByte(21)
  ErrY = NVRAM_ReadByte(22)
  ErrAddr = (CLng(NVRAM_ReadByte(23)) * 256&)
  ErrAddr = ErrAddr + CLng(NVRAM_ReadByte(24))
  
  SINnum = (CLng(NVRAM_ReadByte(25)) * 65536)
  SINnum = SINnum + (CLng(NVRAM_ReadByte(26)) * 256&)
  SINnum = SINnum + CLng(NVRAM_ReadByte(27))
  
  
  ' This is the complete list from the Econet Design and
  ' Installation Guide (June 1993). I suspect some of these
  ' errors may be more Level Two/Three specific, but there
  ' is no marker to indicate this, so until I know otherwise
  ' we'll assume the FileStore *could* return any one of these.
  '
  ' PS: Yes, there appear to be a number of duplications.
  '
  ' FYI: ADFS disc errors are reported as internal errors on
  '      the FileStore (i.e. *FSReport), but numbers listed run
  '      from &00 to &2F; this can't be the same here as we'd
  '      obliterate a bunch of otherwise important codes.
  '      Can anybody help?
  
  ErrMsg = "<code &&" & Hex(ErrCode) & "not known>"
  Select Case ErrCode
    Case &H1: ErrMsg = "Frame size too big"
    Case &H2: ErrMsg = "(Entry) Function Code was zero"
    Case &H3: ErrMsg = "(Entry) Unknown entry code"
    Case &H4: ErrMsg = "(GetVec) Size too big"
    Case &H5: ErrMsg = "(GetVec) Insufficient space"
    Case &H6: ErrMsg = "(GetVec) Size of store requested was zero"
    Case &H7: ErrMsg = "(ModCom) Can't get big buffer"
    Case &H8: ErrMsg = "(GetInt) Not a digit"
    Case &H9: ErrMsg = "Unable to open RxCB (net card bad?)"
    Case &HA: ErrMsg = "(RdBtmp) Disc bitmap error (disc corrupt?)"
    Case &HB: ErrMsg = "Writing to sector zero (disc corrupt)"
    Case &H11: ErrMsg = "Unknown machine"
    Case &H12: ErrMsg = "Too many users logged on"
    Case &H13: ErrMsg = "(UsrMan) Restart called twice!"
    Case &H14: ErrMsg = "Object is not a directory"
    Case &H15: ErrMsg = "User not logged on"
    Case &H16: ErrMsg = "A station has number zero (invalid)"
    Case &H21: ErrMsg = "Can't find ""$.Passwords"""
    Case &H22: ErrMsg = "User identifier not found"
    Case &H23: ErrMsg = "Incorrect password" ' *that* is logged?
    Case &H24: ErrMsg = "Insufficient privilege"
    Case &H25: ErrMsg = "Bad password (starts non-letter or ctrlcodes)"
    Case &H26: ErrMsg = "User identifier already exists"
    Case &H27: ErrMsg = "Password file syntax error"
    Case &H28: ErrMsg = "Password file full"
    Case &H29: ErrMsg = """$.Passwords"" has wrong type (i.e. is dir?)"
    Case &H31: ErrMsg = "(StrMan) Restart called twice!"
    Case &H32: ErrMsg = "SIN=0 (should not occur!)"
    Case &H33: ErrMsg = "RefCount = &FF (should not occur!)"
    Case &H34: ErrMsg = "RefCount = &00 (should not occur!)"
    Case &H35: ErrMsg = "Size of file too big or zero"
    Case &H36: ErrMsg = "Password file not multiple of 256 bytes!"
    Case &H37: ErrMsg = "No free cache descriptors"
    Case &H38: ErrMsg = "Window Ref Count > 0"
    Case &H39: ErrMsg = "Big buffer already in use"
    Case &H3A: ErrMsg = "Invalid buffer address"
    Case &H3B: ErrMsg = "RefCount = &FF (should not occur!)" 'dupe
    Case &H3C: ErrMsg = "Store deadlock! Cache full"
    Case &H3D: ErrMsg = "Arithmetic overflow in TstGap"
    Case &H41: ErrMsg = "CDir too big"
    Case &H42: ErrMsg = "Broken directory (oh balls!)"
    Case &H43: ErrMsg = "Object not found"
    Case &H44: ErrMsg = "Object not a directory"
    Case &H45: ErrMsg = "Insufficient access (no read access)"
    Case &H46: ErrMsg = "Wrong args to OSARGS read/write"
    Case &H47: ErrMsg = "Directory entry locked"
    Case &H49: ErrMsg = "Object in use (i.e. is open)"
    Case &H4A: ErrMsg = "Directory is not empty"
    Case &H4B: ErrMsg = "Types do not match"
    Case &H4C: ErrMsg = "No write access"
    Case &H4D: ErrMsg = "Maximum directory size reached"
    Case &H4E: ErrMsg = "Client asks for too many entries"
    Case &H4F: ErrMsg = "Bad arg to OSARGS examine"
    Case &H51: ErrMsg = "Disc number not found"
    Case &H52: ErrMsg = "Disc space exhausted"
    Case &H53: ErrMsg = "SIN not found for start of chain"
    Case &H54: ErrMsg = "Disc is not a file server disc (!)"
    '    &H55: Both sector maps corrupt (Level II only)
    Case &H56: ErrMsg = "Illegal drive number"
    Case &H57: ErrMsg = "Map sequence numbers differ by > 1"
    Case &H58: ErrMsg = "Illegal object size (i.e. is zero)"
    '    &H59: New map doesn't fit in old space (Level II only)
    Case &H5A: ErrMsg = "Disc of same name already in use!"
    Case &H5B: ErrMsg = "No more space in Map descriptors"
    Case &H5C: ErrMsg = "Insufficient user free space"
    Case &H61: ErrMsg = "(RndMan) Restart called twice!"
    Case &H62: ErrMsg = "Invalid handle "
    Case &H63: ErrMsg = "Handle quota exhausted"
    Case &H64: ErrMsg = "Handle table full"
    Case &H65: ErrMsg = "Object not open"
    Case &H66: ErrMsg = "Copy not for file objects"
    Case &H67: ErrMsg = "Random table full"
    Case &H68: ErrMsg = "File already open"
    Case &H69: ErrMsg = "Object not a file"
    Case &H6A: ErrMsg = "EOF on reading"
    Case &H6B: ErrMsg = "Insufficient access"
    Case &H6C: ErrMsg = "Attempt to point outside of file"
    Case &H6D: ErrMsg = "Invalid arguments to RDSTAR"
    Case &H6E: ErrMsg = "Object not open for update"
    Case &H6F: ErrMsg = "GetByte trying to read after EOF"
    Case &H71: ErrMsg = "Invalid number of sectors (aaargh!)"
    Case &H72: ErrMsg = "Store address overflow"
    Case &H73: ErrMsg = "Accessing beyond EOF"
    Case &H74: ErrMsg = "Invalid SIN (protected area; panic!)"
    Case &H75: ErrMsg = "Disc protected"
    Case &H76: ErrMsg = "Disc error"
    Case &H83: ErrMsg = "Too much data sent from client"
    Case &H84: ErrMsg = "Wait times out"
    Case &H85: ErrMsg = "Invalid function code"
    Case &H87: ErrMsg = "Bad ""*Access"" string"
    Case &H89: ErrMsg = "Bad file title (mismatched quotes)"
    Case &H8A: ErrMsg = "File too big (>16 MeB)"
    Case &H8C: ErrMsg = "Bad access privilege letter (not W, R, or L)"
    Case &H8D: ErrMsg = "Excess data in PutBytes"
    Case &H8E: ErrMsg = "Bad information argument"
    Case &H8F: ErrMsg = "Bad argument to RDAR (Read Obj. Info.)"
    Case &H90: ErrMsg = "Bad date and time"
    Case &HA0: ErrMsg = "Line jammed (no network idle state)"
    Case &HA1: ErrMsg = "Net error (error during transmission)"
    Case &HA2: ErrMsg = "Not listening (client not there/available)"
    Case &HA3: ErrMsg = "No clock (ticky-tock dead? [reality: unplugged])"
    Case &HA4: ErrMsg = "Bad transmit block"
    Case &HA5: ErrMsg = "No reply"
    Case &HAC: ErrMsg = "Bad user name in password file"
    Case &HAE: ErrMsg = "User not logged on"
    Case &HAF: ErrMsg = "Types don't match"
    Case &HB0: ErrMsg = "Renaming across two discs"
    Case &HB1: ErrMsg = "User ID already in password file"
    Case &HB2: ErrMsg = "Password file full (>2097 users!)"
    Case &HB3: ErrMsg = "Maximum dir. size reached (255 entries)"
    Case &HB4: ErrMsg = "Directory not empty (del. files then dir)"
    Case &HB5: ErrMsg = "Trying to load a directory"
    Case &HB6: ErrMsg = "Disc error on map read/write (disc corrupt)"
    Case &HB7: ErrMsg = "Attempt to point outside file"
    Case &HB8: ErrMsg = "User table full (too many users)"
    Case &HB9: ErrMsg = "Syntax error in password"
    Case &HBA: ErrMsg = "Insufficient priv. (not sys user)"
    Case &HBB: ErrMsg = "Incorrect password"
    Case &HBC: ErrMsg = "User ID not found in password file"
    Case &HBD: ErrMsg = "Insufficient access (not sys user)"
    Case &HBE: ErrMsg = "Object not a directory"
    Case &HBF: ErrMsg = "Client ID is not in user table"
    Case &HC0: ErrMsg = "Handle quota exhausted (too many users)"
    Case &HC1: ErrMsg = "File not open for update"
    Case &HC2: ErrMsg = "Object in use (i.e. open)"
    Case &HC3: ErrMsg = "Dir entry locked"
    Case &HC6: ErrMsg = "Disc space exhausted"
    Case &HC7: ErrMsg = "Unrecoverable disc error"
    '    &HC8: Disc number not found (Level II only)
    Case &HC9: ErrMsg = "Disc protected"
    Case &HCC: ErrMsg = "Invalid characters in file name"
    Case &HCF: ErrMsg = "Invalid access string"
    Case &HD4: ErrMsg = "File is write only"
    Case &HD5: ErrMsg = "Object not found"
    Case &HD6: ErrMsg = "Disc name not found"
    Case &HDE: ErrMsg = "Invalid handle / channel"
    Case &HDF: ErrMsg = "End of file"
    Case &HF5: ErrMsg = "Internal bit map cache error"
    Case &HF6: ErrMsg = "Cached bit map holds no free sectors"
    Case &HF8: ErrMsg = "Write error (data read <> data written)"
    Case &HF9: ErrMsg = "Attempting to zero illegal amount of disc"
    Case &HFA: ErrMsg = "Multiple block allocate fails"
    Case &HFC: ErrMsg = "Single block allocate fails"
    Case &HFD: ErrMsg = "Bad file name, string etc" ' gee, so vague!
    Case &HFE: ErrMsg = "Bad command"
  End Select
  
  ' append period if end of message is NOT an '!'
  If (Right$(ErrMsg, 1) <> "!") Then ErrMsg = ErrMsg & "."
  
  ' build up the eventual message
  OutStr = "The stored FS error is:" & vbCrLf & _
    vbTab & "ErrCode" & vbTab & "&" & Hex(ErrCode) & vbCrLf & _
    vbTab & "Address" & vbTab & "&" & Right$("0000" & _
    Hex(ErrAddr), 4) & vbCrLf & _
    vbTab & "SIN" & vbTab & "&" & Right$("000000" & _
    Hex(SINnum), 6) & vbCrLf & _
    vbTab & "X register" & vbTab & "&" & Hex(ErrX) & vbCrLf & _
    vbTab & "Y register" & vbTab & "&" & Hex(ErrY) & vbCrLf & _
    vbCrLf & "This error is:" & vbCrLf & vbTab & ErrMsg & vbCrLf
  
  ' now report it
  MsgBox OutStr, vbInformation + vbOKOnly, "FS error report"
End Sub
